@@ -50,13 +50,9 @@
 Module for data exporting data files.
 """
 
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
 import dxchange.dtype as dt
 import numpy as np
 import os
-import six
 import h5py
 import logging
 import re
@@ -106,7 +102,7 @@ def get_extension(fname):
 
 
 def remove_trailing_digits(text):
-    digit_string = re.search('\d+$', text)
+    digit_string = re.search(r'\d+$', text)
     if digit_string is not None:
         number_of_digits = len(digit_string.group())
         text = ''.join(text[:-number_of_digits])
@@ -163,7 +159,7 @@ def _suggest_new_fname(fname, digit):
 
 
 def _init_write(arr, fname, ext, dtype, overwrite):
-    if not (isinstance(fname, six.string_types)):
+    if not (isinstance(fname, str)):
         fname = 'tmp/data' + ext
     else:
         if not fname.endswith(ext):
@@ -564,4 +560,4 @@ def _normalize_imgstacks(img):
                                    img.shape[1]*img.shape[2],
                                   ), 
                        axis=1,
-                      ).reshape(img.shape[0],1,1)
\ No newline at end of file
+                      ).reshape(img.shape[0],1,1)
